@@ -46,7 +46,8 @@ from functools import partial
 import numpy as np
 from scipy import interpolate as interp
 
-from . import _cycles_support, sift, spectra, utils
+from . import _cycles_support, imftools, spectra
+from ._sift_core import _find_extrema
 from .support import (ensure_1d_with_singleton, ensure_2d, ensure_equal_dims,
                       ensure_vector)
 
@@ -71,7 +72,7 @@ def get_cycle_inds(*args, **kwargs):
 
 def get_cycle_vector(phase, return_good=True, mask=None,
                      imf=None, phase_step=1.5 * np.pi,
-                     phase_edge=np.pi / 12):
+                     phase_edge=np.pi / 12, min_len=2):
     """Identify cycles within a instantaneous phase time-course.
 
     Cycles are identified by large phase jumps and can optionally be tested to
@@ -112,7 +113,7 @@ def get_cycle_vector(phase, return_good=True, mask=None,
 
     Good cycles can be idenfied with:
 
-    >>> good_cycles = emd.utils.get_cycle_vector(phase)
+    >>> good_cycles = emd.cycles.get_cycle_vector(phase)
 
     The total number of cycles is then
 
@@ -144,7 +145,7 @@ def get_cycle_vector(phase, return_good=True, mask=None,
 
     if phase.max() > 2 * np.pi:
         print('Wrapping phase')
-        phase = utils.wrap_phase(phase)
+        phase = imftools.wrap_phase(phase)
 
     cycles = np.zeros_like(phase, dtype=int) - 1
 
@@ -175,7 +176,8 @@ def get_cycle_vector(phase, return_good=True, mask=None,
             cycle_phase = phase[inds[jj]:inds[jj + 1], ii]
 
             if return_good:
-                cycle_checks = is_good(cycle_phase, ret_all_checks=True, phase_edge=phase_edge)
+                cycle_checks = is_good(cycle_phase, ret_all_checks=True,
+                                       phase_edge=phase_edge, min_len=min_len)
             else:
                 # Pretend eveything is ok
                 cycle_checks = np.ones((4,), dtype=bool)
@@ -260,8 +262,8 @@ def get_cycle_vector_from_waveform(imf, cycle_start='peaks'):
 
     cycles = np.zeros_like(imf)
     for ii in range(imf.shape[1]):
-        peak_loc, peak_mag = sift._find_extrema(imf[:, ii])
-        trough_loc, trough_mag = sift._find_extrema(-imf[:, ii])
+        peak_loc, peak_mag = _find_extrema(imf[:, ii])
+        trough_loc, trough_mag = _find_extrema(-imf[:, ii])
         trough_mag = -trough_mag
 
         for jj in range(len(peak_loc)-1):
@@ -293,13 +295,17 @@ def get_cycle_vector_from_waveform(imf, cycle_start='peaks'):
     return cycles.astype(int)
 
 
-def is_good(phase, waveform=None, ret_all_checks=False, phase_edge=np.pi/12, mode='cycle'):
+def is_good(phase, waveform=None, ret_all_checks=False, phase_edge=np.pi/12, mode='cycle', min_len=2):
     """Run a set of phase checks to check if a cycle is 'good' or 'bad'.
 
+    This implements the checks defined in [1]_ and and one additional length
+    check. Cycles meeting these criterial have a good chance of providing an
+    interpretable instantaneous frequency estimate.
+
     Parameters
     ----------
     phase : ndarray
-        Phase of the cycle to be checked
+        Instantaneous Phase of the cycle to be checked
     waveform : ndarray
         Optional time-domain waveform to enable control point checks
     ret_all_checks
@@ -308,6 +314,8 @@ def is_good(phase, waveform=None, ret_all_checks=False, phase_edge=np.pi/12, mod
         Maximum distance from 0 or 2pi for the first and last phase value in a
         good cycle. Only used when return_good is True
         (Default value = np.pi/12)
+    min_len : int
+        Minimum length in samples for a valid single cycle.
 
     Returns
     -------
@@ -315,6 +323,16 @@ def is_good(phase, waveform=None, ret_all_checks=False, phase_edge=np.pi/12, mod
         Flag indicating whether cycle is good (or array of booleans
         corresponding to each check.
 
+    Notes
+    -----
+    A good cycle is defined as:
+
+    * having a phase with a strictly positive differential (i.e., no phase reversals)
+    * starting with a phase value 0 ≤ x ≤ phase_edge
+    * ending within 2pi − phase_edge ≤ x ≤ 2π
+    * having four unique control points (peak, trough, ascending edge, and descending edge)
+    * exceeding a minimum length in samples
+
     References
     ----------
     .. [1] Andrew J. Quinn, Vitor Lopes-dos-Santos, Norden Huang, Wei-Kuang
@@ -324,7 +342,7 @@ def is_good(phase, waveform=None, ret_all_checks=False, phase_edge=np.pi/12, mod
            https://doi.org/10.1101/2021.04.12.439547
 
     """
-    cycle_checks = np.zeros((4,), dtype=bool)
+    cycle_checks = np.zeros((5,), dtype=bool)
 
     if mode == 'augmented':
         phase = np.unwrap(phase) - 2*np.pi
@@ -348,9 +366,9 @@ def is_good(phase, waveform=None, ret_all_checks=False, phase_edge=np.pi/12, mod
         # Check we find 5 sensible control points if imf is provided
         try:
             # Should extend this to cope with multiple peaks etc
-            ctrl = (0, sift._find_extrema(waveform)[0][0],
+            ctrl = (0, _find_extrema(waveform)[0][0],
                     np.where(np.gradient(np.sign(waveform)) == -1)[0][0],
-                    sift._find_extrema(-waveform)[0][0],
+                    _find_extrema(-waveform)[0][0],
                     len(waveform))
             if len(ctrl) == 5 and np.all(np.sign(np.diff(ctrl))):
                 cycle_checks[3] = True
@@ -361,6 +379,10 @@ def is_good(phase, waveform=None, ret_all_checks=False, phase_edge=np.pi/12, mod
         # No time-series so assume everything is fine
         cycle_checks[3] = True
 
+    # Check we exceed a minimum length
+    if len(phase) > min_len:
+        cycle_checks[4] = True
+
     if ret_all_checks:
         return cycle_checks
     else:
@@ -457,9 +479,12 @@ def get_chain_stat(chains, var, func=np.mean):
     return stat
 
 
-def phase_align(ip, x, cycles=None, npoints=48, interp_kind='linear', ii=None, mode='cycle'):
-    """
-    Compute phase alignment of a vector of observed values across a set of cycles [1]_.
+def phase_align(ip, x, cycles=None, npoints=48, interp_kind='linear', min_len=1, mode='cycle'):
+    """Align a vector of observations to a template phase time-course.
+
+    This implements the phase alignment method introduced in [1]_. Individual
+    cycles must be longer than 2 samples to be phase-aligned - if a cycle
+    cannot be phase aligned it's output will be set to np.nan.
 
     Parameters
     ----------
@@ -474,6 +499,12 @@ def phase_align(ip, x, cycles=None, npoints=48, interp_kind='linear', ii=None, m
     interp_kind : {'linear','nearest','zero','slinear', 'quadratic','cubic','previous', 'next'}
         Type of interpolation to perform. Argument is passed onto
         scipy.interpolate.interp1d. (Default = 'linear')
+    min_len : int
+        Minimum length in samples for a cycle to be phase aligned. Shorter
+        cycles will be returned as nans.
+    mode : {'cycle', 'augmented'}
+        Whether to phase align a standard 'cycle' or an 'augmented' cycle
+        including a 5th quadrant.
 
     Returns
     -------
@@ -515,11 +546,9 @@ def phase_align(ip, x, cycles=None, npoints=48, interp_kind='linear', ii=None, m
     msg = 'aligning {0} cycles over {1} phase points with {2} interpolation'
     logger.debug(msg.format(cycles.niters, npoints, interp_kind))
 
-    avg = np.zeros((npoints, cycles.niters))
+    avg = np.zeros((npoints, cycles.niters)) * np.nan
     for cind, cycle_inds in cycles:
-        if (ii is not None) and (cind is not ii):
-            continue
-        if cycle_inds is None:
+        if (cycle_inds is None) or (len(cycle_inds) <= min_len):
             continue
         phase_data = ip[cycle_inds].copy()
 
@@ -533,6 +562,8 @@ def phase_align(ip, x, cycles=None, npoints=48, interp_kind='linear', ii=None, m
 
         avg[:, cind] = f(phase_bins)
 
+    logger.debug('{0} cycles not aligned'.format(np.isnan(avg[0, :]).sum()))
+
     logger.info('COMPLETED: phase-align cycles')
     return avg, phase_bins
 
@@ -952,7 +983,7 @@ def cf_end_value(x):
 
 def cf_peak_sample(x, interp=True):
     """Compute index of peak in a single cycle."""
-    locs, pks = sift._find_extrema(x, parabolic_extrema=interp)
+    locs, pks = _find_extrema(x, parabolic_extrema=interp)
     if len(pks) == 0:
         return None
     else:
@@ -961,7 +992,7 @@ def cf_peak_sample(x, interp=True):
 
 def cf_peak_value(x, interp=True):
     """Compute value at peak in a single cycle."""
-    locs, pks = sift._find_extrema(x, parabolic_extrema=interp)
+    locs, pks = _find_extrema(x, parabolic_extrema=interp)
     if len(pks) == 0:
         return None
     else:
@@ -970,7 +1001,7 @@ def cf_peak_value(x, interp=True):
 
 def cf_trough_sample(x, interp=True):
     """Compute index of trough in a single cycle."""
-    locs, trs = sift._find_extrema(-x, parabolic_extrema=interp)
+    locs, trs = _find_extrema(-x, parabolic_extrema=interp)
     trs = -trs
     if len(trs) == 0:
         return None
@@ -980,7 +1011,7 @@ def cf_trough_sample(x, interp=True):
 
 def cf_trough_value(x, interp=True):
     """Compute value at trough in a single cycle."""
-    locs, trs = sift._find_extrema(-x, parabolic_extrema=interp)
+    locs, trs = _find_extrema(-x, parabolic_extrema=interp)
     trs = -trs
     if len(trs) == 0:
         return None
@@ -1145,7 +1176,7 @@ class Cycles:
 
     """
 
-    def __init__(self, IP, phase_step=1.5 * np.pi, phase_edge=np.pi / 12,
+    def __init__(self, IP, phase_step=1.5 * np.pi, phase_edge=np.pi/12, min_len=2,
                  compute_timings=False, mode='cycle', use_cache=True):
         """Class storing and manipulating singl cycles."""
         logger.info('Initialising Cycles')
@@ -1173,7 +1204,7 @@ class Cycles:
         self.mask_conditions = None
 
         self.metrics = dict()
-        good_func = partial(is_good, phase_edge=phase_edge)
+        good_func = partial(is_good, phase_edge=phase_edge, min_len=min_len)
         self.compute_cycle_metric('is_good', self.phase, good_func, dtype=int)
         if compute_timings:
             self.compute_cycle_timings()
