@@ -301,6 +301,9 @@ def wrap_verbose(func):
             tmp_level = kwargs['verbose']
             current_level = get_level()
             set_level(level=tmp_level)
+        elif ('verbose' in kwargs) and (kwargs['verbose'] is None):
+            # Don't do anything
+            pass
         elif ('verbose' in kwargs):
             logger.warning("Logger level '{0}' not recognised - level is unchanged".format(kwargs['verbose']))
 
