@@ -30,7 +30,8 @@ import numpy as np
 import sparse
 from scipy import ndimage, signal
 
-from . import cycles, utils
+from . import cycles, imftools
+from ._sift_core import interp_envelope
 from .support import ensure_2d, ensure_equal_dims, ensure_vector
 
 # Housekeeping for logging
@@ -101,7 +102,7 @@ def frequency_transform(imf, sample_rate, method, smooth_freq=3,
     elif method == 'nht':
         logger.info('Using Amplitude-Normalised Hilbert transform')
 
-        n_imf = utils.amplitude_normalise(imf)
+        n_imf = imftools.amplitude_normalise(imf)
         analytic_signal = signal.hilbert(n_imf, axis=0)
 
         orig_dim = imf.ndim
@@ -112,8 +113,8 @@ def frequency_transform(imf, sample_rate, method, smooth_freq=3,
         iamp = np.zeros_like(imf)
         for ii in range(imf.shape[1]):
             for jj in range(imf.shape[2]):
-                iamp[:, ii, jj] = utils.interp_envelope(imf[:, ii, jj],
-                                                        mode='upper')
+                iamp[:, ii, jj] = interp_envelope(imf[:, ii, jj],
+                                                  mode='upper')
         if orig_dim == 2:
             iamp = iamp[:, :, 0]
 
@@ -137,8 +138,8 @@ def frequency_transform(imf, sample_rate, method, smooth_freq=3,
         iamp = np.zeros_like(imf)
         for ii in range(imf.shape[1]):
             for jj in range(imf.shape[2]):
-                iamp[:, ii, jj] = utils.interp_envelope(imf[:, ii, jj],
-                                                        mode='upper')
+                iamp[:, ii, jj] = interp_envelope(imf[:, ii, jj],
+                                                  mode='upper')
 
         if orig_dim == 2:
             iamp = iamp[:, :, 0]
@@ -157,8 +158,8 @@ def frequency_transform(imf, sample_rate, method, smooth_freq=3,
         iamp = np.zeros_like(imf)
         for ii in range(imf.shape[1]):
             for jj in range(imf.shape[2]):
-                iamp[:, ii, jj] = utils.interp_envelope(imf[:, ii, jj],
-                                                        mode='upper')
+                iamp[:, ii, jj] = interp_envelope(imf[:, ii, jj],
+                                                  mode='upper')
 
         if orig_dim == 2:
             iamp = iamp[:, :, 0]
@@ -167,12 +168,12 @@ def frequency_transform(imf, sample_rate, method, smooth_freq=3,
         logger.info('Using Direct-Quadrature transform')
         raise ValueError('direct_quad method is broken!')
 
-        n_imf = utils.amplitude_normalise(imf.copy())
+        n_imf = imftools.amplitude_normalise(imf.copy())
         iphase = np.unwrap(phase_angle(n_imf))
 
         iamp = np.zeros_like(imf)
         for ii in range(imf.shape[1]):
-            iamp[:, ii] = utils.interp_envelope(imf[:, ii, None], mode='combined')
+            iamp[:, ii] = interp_envelope(imf[:, ii, None], mode='combined')
 
     else:
         logger.error("Method '{0}' not recognised".format(method))
@@ -188,7 +189,7 @@ def frequency_transform(imf, sample_rate, method, smooth_freq=3,
     ifreq = freq_from_phase(iphase, sample_rate, savgol_width=smooth_freq)
 
     # Return wrapped phase
-    iphase = utils.wrap_phase(iphase)
+    iphase = imftools.wrap_phase(iphase)
 
     logger.info('COMPLETED: compute frequency stats. Returning {0} imfs'.format(iphase.shape[1]))
     return iphase, ifreq, iamp
@@ -221,7 +222,7 @@ def quadrature_transform(X, fix_zerocrossings=False):
        1(2), 177–229. https://doi.org/10.1142/s1793536909000096
 
     """
-    nX = utils.amplitude_normalise(X.copy(), clip=False)
+    nX = imftools.amplitude_normalise(X.copy(), clip=False)
     nX = nX / (np.abs(nX).max() + 1e-8)  # Clip any remaining points outside -1 < x < 1
 
     # Avoid occasional 'invalid value encountered' RuntimeWarning in sqrt using
@@ -331,7 +332,7 @@ def phase_from_complex_signal(complex_signal, smoothing=None,
         iphase = iphase + np.pi
 
     if ret_phase == 'wrapped':
-        return utils.wrap_phase(iphase)
+        return imftools.wrap_phase(iphase)
     elif ret_phase == 'unwrapped':
         return iphase
 
@@ -541,7 +542,7 @@ def hilberthuang(IF, IA, edges=None,
 
     Notes
     -----
-    Run a HHT with an automatically genereated set of histogram bins:
+    Run a HHT with an automatically generated set of histogram bins:
 
     >>> f, hht = emd.spectra.hilberthuang(IF, IA, sample_rate=512)
 
@@ -567,7 +568,7 @@ def hilberthuang(IF, IA, edges=None,
     >>> f, hht = emd.spectra.hilberthuang(IF, IA, edges=(1, 50, 49), sample_rate=512, sum_time=False)
 
     Run a HHT and return a memory efficient sparse array - this is strongly
-    reccommended for very large HHTs
+    recommended for very large HHTs
 
     >>> f, hht = emd.spectra.hilberthuang(IF, IA, edges=(1, 50, 49), sample_rate=512,
     >>>                                   sum_time=False, return_sparse=True)
@@ -693,8 +694,8 @@ def holospectrum(IF, IF2, IA2,
         Holospectrum of input data.
 
     Notes
-    ------
-    Run a Holospectrum with an automatically genereated set of histogram bins:
+    -----
+    Run a Holospectrum with an automatically generated set of histogram bins:
 
     >>> fcarrier, fam, holo = emd.spectra.holospectrum(IF, IA, sample_rate=512)
 
@@ -712,7 +713,7 @@ def holospectrum(IF, IF2, IA2,
     >>>                                                edges2=(0.1, 20, 32, 'log'))
 
     Run a Holospectrum without summing over the time dimensions and return the
-    result in a memory efficient sparse array - this is strongly reccommended
+    result in a memory efficient sparse array - this is strongly recommended
     for very large HHTs
 
     >>> fcarrier, fam, holo = emd.spectra.holospectrum(IF, IA, sample_rate=512,
@@ -769,39 +770,117 @@ def holospectrum(IF, IF2, IA2,
     return bins, bins2, holo
 
 
-def hilbertmarginal(IF, IA, edges=None, bins2=None, edges2=None, order=2,
-                    sample_rate=1, return_sparse=False):
+def hilbertmarginal(IF, IA, order=2,
+                    freq_edges=None, amp_edges=None,
+                    sum_time=True, sum_imfs=True,
+                    sample_rate=1, scaling=None,
+                    return_sparse=False,
+                    return_Gb_limit=10):
     """Compute a generalised Hilbert marginal spectrum.
 
-    This is an experimental function which possibly implements the method
-    introduced in https://arxiv.org/abs/1401.4211
+    This is an experimental function which probably implements the method
+    introduced in Huang et al (2008) _[1]. This creates a 2D
+    amplitude-frequency representation of the signal.
+
+    Parameters
+    ----------
+    IF : ndarray
+        2D first level instantaneous frequencies
+    IA : ndarray
+        2D first level instantaneous amplitudes
+    order : int
+        Power to which amplitude is raised before spectrum computation.
+    freq_edges: {ndarray, tuple or None}
+        Definition of the frequency bins used in the spectrum. This may be:
+
+        * array_like vector of bin edge values (as defined by
+        emd.spectra.define_hist_bins)
+
+        * a tuple of values that can be passed to emd.spectra.define_hist_bins
+        (eg edges=(1,50,49) will define 49 bins between 1 and 50Hz)
+
+        * None in which case a sensible set of bins will be defined from the
+        input data (this is the default option)
+    amp_edges : {ndarray, tuple or None}
+        Definition of amplitude bins used in spectrum. Format options are the
+        same as for `freq_edges`.
+    sum_time : boolean
+        Flag indicating whether to sum across time dimension
+    sum_imfs : boolean
+        Flag indicating whether to sum across IMF dimension
+    sample_rate : float
+        Sampling rate of the data used in 'density' scaling
+    scaling : {'density', 'spectrum', None}
+        Switch specifying the normalisation or scaling applied to the spectrum.
+    return_sparse : bool
+         Flag indicating whether to return the full or sparse form(Default value = True)
+    return_Gb_limit : {float, None}
+        Maximum array size in Gb that will be returned if a non-sparse/dense
+        array is being returned (default = 10). If the function return would
+        exceed this size, the function will raise an error. If set to None,
+        then no limit is imposed. Sparse arrays are always returned.
+
+    Returns
+    -------
+    a : ndarray
+        Vector of histogram bin centers for each amplitude
+    f : ndarray
+        Vector of histogram bin centers for each frequency
+    hht : ndarray
+        2D array containing the Hilbert-Huang Transform
+
+    References
+    ----------
+    .. [1] Huang, Y. X., Schmitt, F. G., Lu, Z. M., & Liu, Y. L. (2008). An
+       amplitude-frequency study of turbulent scaling intermittency using
+       Empirical Mode Decomposition and Hilbert Spectral Analysis. In EPL
+       (Europhysics Letters) (Vol. 84, Issue 4, p. 40010). IOP Publishing.
+       https://doi.org/10.1209/0295-5075/84/40010
 
     """
-    logger.warning('hilbertmarginal is an experimental function - Caveat emptor!')
+    logger.info('STARTED: compute Hilbert-Marginal spectrum')
 
-    edges, bins = _histogram_bin_relay(edges, IF.flatten())
-    edges2, bins2 = _histogram_bin_relay(edges2, IA.flatten())
+    IF, IA = ensure_2d([IF, IA], ['IF', 'IA'], 'hilberthuang')
+    ensure_equal_dims((IF, IA), ('IF', 'IA'), 'hilberthuang')
 
-    hima = _higher_order_spectra(IF,
-                                 IA[:, :, None],
-                                 np.ones_like(IA[:, :,  None]),
-                                 edges, edges2)
+    freq_edges, freq_bins = _histogram_bin_relay(freq_edges, IF.flatten())
+    logger.debug('Freq bins: {0} to {1} in {2} steps'.format(freq_edges[0],
+                                                             freq_edges[-1],
+                                                             len(freq_edges)))
 
-    # Create PDF histogram
-    hima = hima.sum(axis=(2, 3))
-    hima = hima / IF.size
+    amp_edges, amp_bins = _histogram_bin_relay(amp_edges, IA.flatten())
+    logger.debug('Amp bins: {0} to {1} in {2} steps'.format(amp_edges[0],
+                                                            amp_edges[-1],
+                                                            len(amp_edges)))
 
-    dA = np.diff(bins2)[0]
-    A_values = bins2[None, :, None, None]**order
+    # Compute HOS - distribution of amplitude across amplitude, frequency,
+    # time and IMF
+    hima = _higher_order_spectra(IA,
+                                 IF[:, :, None],
+                                 IA[:, :,  None],
+                                 amp_edges, freq_edges)
+    # hima is a spase array of dimensions
+    # [len(edges), len(edges2), num_samples, num_imfs, 1]
+
+    # Get amplitude values in broadcastable shape
+    A_values = np.reshape(IA, (1, 1, IA.shape[0], IA.shape[1], 1))**order
+
+    # Scale hima by amplitude values and resolution
+    dA = np.diff(amp_bins)[0]
     hima = hima * A_values * dA
 
-    hima = _post_process_spectra(hima, 2,
-                                 sum_time=False,
-                                 sum_imfs=False,
-                                 mode=None,
-                                 return_sparse=return_sparse)
+    # Create PDF histogram
+    sum_dims = np.where([0, 0, sum_time, sum_imfs, 1])[0]
+    hima = hima.sum(axis=sum_dims)
+    hima = hima / IF.size
 
-    return hima
+    # Post-process - summing as already been done
+    hima = _post_process_spectra(hima, mode=None, sample_rate=sample_rate,
+                                 scaling=scaling, return_sparse=return_sparse,
+                                 return_Gb_limit=return_Gb_limit)
+
+    logger.info('COMPLETED: Hilbert-Marginal Spectrum - output size {0}'.format(hima.shape))
+    return amp_bins, freq_bins, hima
 
 
 def _post_process_spectra(spec, sum_dims=None,
@@ -1189,7 +1268,6 @@ def _compute_centres_from_edges(edges, method='mean'):
     if method == 'geometric':
         bins = np.sqrt(edges[1:] * edges[:-1])
     elif method == 'mean':
-        print(edges)
         bins = (edges[1:] + edges[:-1]) / 2
     else:
         msg = 'method \'{0}\' not recognised. please use \'mean\' or \'geometric\'.'
