@@ -23,11 +23,11 @@ Errors:
 
 import logging
 import os
+import pathlib
 
 import numpy as np
-import pkg_resources  # part of setuptoos
-
-from . import sift
+import pkg_resources  # part of setuptools
+from joblib import Parallel, delayed
 
 # Housekeeping for logging
 logger = logging.getLogger(__name__)
@@ -35,7 +35,8 @@ logger = logging.getLogger(__name__)
 
 def get_install_dir():
     """Get directory path of currently installed & imported emd."""
-    return os.path.dirname(sift.__file__)
+    #return os.path.dirname(sift.__file__)
+    return str(pathlib.Path(__file__).parent.resolve())
 
 
 def get_installed_version():
@@ -71,6 +72,19 @@ def run_tests():
             logger.info('EMD package tests passed')
 
 
+# Parallel processing
+
+def run_parallel(pfunc, args, nprocesses=1):
+    """Run set of processes in serial or parallel."""
+    if nprocesses > 1:
+        with Parallel(n_jobs=nprocesses) as parallel:
+            res = parallel(delayed(pfunc)(*aa) for aa in args)
+    else:
+        res = [pfunc(*aa) for aa in args]
+
+    return res
+
+
 # Ensurance Department
 
 
